@@ -9,7 +9,6 @@ import java.net.URI;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
@@ -17,6 +16,7 @@ import java.util.List;
 import java.util.TimeZone;
 import java.util.stream.Collectors;
 
+import org.apache.commons.lang3.StringUtils;
 import org.dogtagpki.acme.ACMEAccount;
 import org.dogtagpki.acme.ACMEAuthorization;
 import org.dogtagpki.acme.ACMECertificate;
@@ -26,10 +26,11 @@ import org.dogtagpki.acme.ACMENonce;
 import org.dogtagpki.acme.ACMEOrder;
 import org.dogtagpki.acme.JWK;
 
+import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmscore.base.FileConfigStore;
-import com.netscape.cmscore.base.PropConfigStore;
 import com.netscape.cmscore.ldapconn.LDAPConfig;
 import com.netscape.cmscore.ldapconn.LdapBoundConnFactory;
+import com.netscape.cmscore.ldapconn.PKISocketConfig;
 import com.netscape.cmsutil.password.IPasswordStore;
 import com.netscape.cmsutil.password.PlainPasswordFile;
 
@@ -65,6 +66,7 @@ public class LDAPDatabase extends ACMEDatabase {
 
     public static org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(LDAPDatabase.class);
 
+    static final String RDN_CONFIG = "ou=config";
     static final String RDN_NONCE = "ou=nonces";
     static final String RDN_ACCOUNT = "ou=accounts";
     static final String RDN_ORDER = "ou=orders";
@@ -80,6 +82,7 @@ public class LDAPDatabase extends ACMEDatabase {
     static final String ATTR_AUTHORIZATION_WILDCARD = "acmeAuthorizationWildcard";
     static final String ATTR_CERTIFICATE_ID = "acmeCertificateId";
     static final String ATTR_CHALLENGE_ID = "acmeChallengeId";
+    static final String ATTR_CREATED = "acmeCreated";
     static final String ATTR_ERROR = "acmeError";
     static final String ATTR_EXPIRES = "acmeExpires";
     static final String ATTR_IDENTIFIER = "acmeIdentifier";
@@ -87,8 +90,9 @@ public class LDAPDatabase extends ACMEDatabase {
     static final String ATTR_ORDER_ID = "acmeOrderId";
     static final String ATTR_STATUS = "acmeStatus";
     static final String ATTR_TOKEN = "acmeToken";
-    static final String ATTR_USER_CERTIFICATE = "userCertificate";
+    static final String ATTR_USER_CERTIFICATE = "userCertificate;binary";
     static final String ATTR_VALIDATED_AT = "acmeValidatedAt";
+    static final String ATTR_ENABLED = "acmeEnabled";
 
     static final String OBJ_ACCOUNT = "acmeAccount";
     static final String OBJ_AUTHORIZATION = "acmeAuthorization";
@@ -101,8 +105,6 @@ public class LDAPDatabase extends ACMEDatabase {
 
     static final String IDENTIFIER_TYPE_DNS = "dns";
 
-    static final List<String> IDENTIFIER_TYPES = Arrays.asList(IDENTIFIER_TYPE_DNS);
-
     // The LDAP Generalized Time syntax
     static final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmssZ");
 
@@ -113,13 +115,16 @@ public class LDAPDatabase extends ACMEDatabase {
     enum LoadChallenges { DoLoad , DontLoad };
     enum OnNoSuchObject { Ignore , Throw };
 
-    String basedn;
+    String baseDN;
 
     LdapBoundConnFactory connFactory = null;
 
+    Boolean enabled;
+    LDAPConfigMonitor monitor;
+
     public void init() throws Exception {
 
-        PropConfigStore cs;
+        EngineConfig cs;
         IPasswordStore ps;
         LDAPConfig ldapConfig;
 
@@ -129,7 +134,7 @@ public class LDAPDatabase extends ACMEDatabase {
 
             logger.info("Loading LDAP database configuration from database.conf");
 
-            cs = new PropConfigStore();
+            cs = new EngineConfig();
             ps = new PlainPasswordFile();
             ldapConfig = new LDAPConfig(null);
 
@@ -208,33 +213,99 @@ public class LDAPDatabase extends ACMEDatabase {
 
             logger.info("Loading LDAP database configuration from " + configFile);
 
-            cs = new PropConfigStore(new FileConfigStore(configFile));
+            cs = new EngineConfig(new FileConfigStore(configFile));
             cs.load();
 
-            ps = IPasswordStore.getPasswordStore("acme", cs.getProperties());
-            ldapConfig = cs.getSubStore("internaldb", LDAPConfig.class);
+            ps = IPasswordStore.create(cs.getPasswordStoreConfig());
+            ldapConfig = cs.getInternalDBConfig();
         }
 
-        basedn = config.getParameter("basedn");
-        if (basedn == null) {
-            basedn = config.getParameter("baseDN");
+        baseDN = config.getParameter("basedn");
+        if (baseDN == null) {
+            baseDN = config.getParameter("baseDN");
         } else {
             logger.warn("The basedn parameter has been deprecated. Use baseDN instead.");
         }
-        logger.info("- base DN: " + basedn);
+        logger.info("- base DN: " + baseDN);
+
+        PKISocketConfig socketConfig = cs.getSocketConfig();
 
         connFactory = new LdapBoundConnFactory("acme");
-        connFactory.init(cs, ldapConfig, ps);
+        connFactory.init(socketConfig, ldapConfig, ps);
+
+        String monitorEnabled = config.getParameter("monitor.enabled");
+        logger.info("- monitor enabled: " + monitorEnabled);
+
+        if ("true".equals(monitorEnabled)) {
+
+            monitor = new LDAPConfigMonitor();
+            monitor.setDatabase(this);
+
+            new Thread(monitor, "LDAPConfigMonitor").start();
+        }
+    }
+
+    public Boolean getEnabled() throws Exception {
+
+        // If monitor is not enabled, get config from database on each request.
+        // Otherwise, return the config stored in memory which is updated by
+        // LDAPConfigMonitor.
+
+        if (monitor == null) {
+
+            String dn = RDN_CONFIG + "," + baseDN;
+            LDAPEntry entry = ldapGet(dn);
+
+            if (entry == null) {
+                enabled = null;
+                return null;
+            }
+
+            LDAPAttribute acmeEnabled = entry.getAttribute(ATTR_ENABLED);
+
+            if (acmeEnabled == null) {
+                enabled = null;
+                return enabled;
+            }
+
+            String value = acmeEnabled.getStringValueArray()[0];
+            enabled = Boolean.parseBoolean(value);
+        }
+
+        return enabled;
+    }
+
+    public void setEnabled(Boolean enabled) throws Exception {
+
+        // always update config both in database and memory
+
+        String dn = RDN_CONFIG + "," + baseDN;
+
+        LDAPModificationSet mods = new LDAPModificationSet();
+        if (enabled == null) {
+            LDAPAttribute acmeEnabled = new LDAPAttribute(ATTR_ENABLED);
+            mods.add(LDAPModification.DELETE, acmeEnabled);
+        } else {
+            LDAPAttribute acmeEnabled = new LDAPAttribute(ATTR_ENABLED, enabled ? "TRUE" : "FALSE");
+            mods.add(LDAPModification.REPLACE, acmeEnabled);
+        }
+
+        ldapModify(dn, mods);
+
+        this.enabled = enabled;
     }
 
     public ACMENonce getNonce(String nonceID) throws Exception {
-        String dn = ATTR_NONCE_ID +  "=" + nonceID + "," + RDN_NONCE + "," + basedn;
+        String dn = ATTR_NONCE_ID +  "=" + nonceID + "," + RDN_NONCE + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
 
         ACMENonce nonce = new ACMENonce();
         nonce.setID(nonceID);
 
+        LDAPAttribute attrCreated = entry.getAttribute(ATTR_CREATED);
+        nonce.setCreationTime(dateFormat.parse(attrCreated.getStringValues().nextElement()));
+
         LDAPAttribute attrExpires = entry.getAttribute(ATTR_EXPIRES);
         nonce.setExpirationTime(dateFormat.parse(attrExpires.getStringValues().nextElement()));
 
@@ -245,10 +316,11 @@ public class LDAPDatabase extends ACMEDatabase {
         LDAPAttribute[] attrs = {
             new LDAPAttribute(ATTR_OBJECTCLASS, OBJ_NONCE),
             new LDAPAttribute(ATTR_NONCE_ID, nonce.getID()),
+            new LDAPAttribute(ATTR_CREATED, dateFormat.format(nonce.getCreationTime())),
             new LDAPAttribute(ATTR_EXPIRES, dateFormat.format(nonce.getExpirationTime()))
         };
         LDAPAttributeSet attrSet = new LDAPAttributeSet(attrs);
-        String dn = ATTR_NONCE_ID + "=" + nonce.getID() + "," + RDN_NONCE + "," + basedn;
+        String dn = ATTR_NONCE_ID + "=" + nonce.getID() + "," + RDN_NONCE + "," + baseDN;
         LDAPEntry entry = new LDAPEntry(dn, attrSet);
         ldapAdd(entry);
     }
@@ -257,7 +329,7 @@ public class LDAPDatabase extends ACMEDatabase {
         ACMENonce nonce = getNonce(nonceID);
         if (nonce == null) return null;
 
-        String dn = ATTR_NONCE_ID + "=" + nonceID + "," + RDN_NONCE + "," + basedn;
+        String dn = ATTR_NONCE_ID + "=" + nonceID + "," + RDN_NONCE + "," + baseDN;
         ldapDelete(dn, OnNoSuchObject.Ignore);
 
         return nonce;
@@ -266,7 +338,7 @@ public class LDAPDatabase extends ACMEDatabase {
     public void removeExpiredNonces(Date currentTime) throws Exception {
         String[] attrs = {"1.1"};  // suppress attrs for performance; we only need DN
         List<LDAPEntry> entries = ldapSearch(
-            RDN_NONCE + "," + basedn,
+            RDN_NONCE + "," + baseDN,
             "(" + ATTR_EXPIRES + "<=" + dateFormat.format(currentTime) + ")",
             attrs
         );
@@ -276,14 +348,17 @@ public class LDAPDatabase extends ACMEDatabase {
     }
 
     public ACMEAccount getAccount(String accountID) throws Exception {
-        String dn = ATTR_ACCOUNT_ID + "=" + accountID + "," + RDN_ACCOUNT + "," + basedn;
+        String dn = ATTR_ACCOUNT_ID + "=" + accountID + "," + RDN_ACCOUNT + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
 
         ACMEAccount account = new ACMEAccount();
         account.setID(accountID);
 
-        LDAPAttribute attr = entry.getAttribute(ATTR_STATUS);
+        LDAPAttribute attr = entry.getAttribute(ATTR_CREATED);
+        account.setCreationTime(dateFormat.parse(attr.getStringValues().nextElement()));
+
+        attr = entry.getAttribute(ATTR_STATUS);
         account.setStatus(attr.getStringValues().nextElement());
 
         attr = entry.getAttribute(ATTR_ACCOUNT_KEY);
@@ -304,6 +379,7 @@ public class LDAPDatabase extends ACMEDatabase {
         LDAPAttribute[] attrs = {
             new LDAPAttribute(ATTR_OBJECTCLASS, OBJ_ACCOUNT),
             new LDAPAttribute(ATTR_ACCOUNT_ID, account.getID()),
+            new LDAPAttribute(ATTR_CREATED, dateFormat.format(account.getCreationTime())),
             new LDAPAttribute(ATTR_ACCOUNT_KEY, account.getJWK().toJSON()),
             new LDAPAttribute(ATTR_STATUS, account.getStatus())
         };
@@ -313,7 +389,7 @@ public class LDAPDatabase extends ACMEDatabase {
             attrSet.add(new LDAPAttribute(ATTR_ACCOUNT_CONTACT, contacts));
         }
 
-        String dn = ATTR_ACCOUNT_ID + "=" + account.getID() + "," + RDN_ACCOUNT + "," + basedn;
+        String dn = ATTR_ACCOUNT_ID + "=" + account.getID() + "," + RDN_ACCOUNT + "," + baseDN;
         LDAPEntry entry = new LDAPEntry(dn, attrSet);
         ldapAdd(entry);
     }
@@ -323,7 +399,7 @@ public class LDAPDatabase extends ACMEDatabase {
      * and just perform the update.
      */
     public void updateAccount(ACMEAccount account) throws Exception {
-        String dn = ATTR_ACCOUNT_ID + "=" + account.getID() + "," + RDN_ACCOUNT + "," + basedn;
+        String dn = ATTR_ACCOUNT_ID + "=" + account.getID() + "," + RDN_ACCOUNT + "," + baseDN;
         LDAPModificationSet mods = new LDAPModificationSet();
         mods.add(
             LDAPModification.REPLACE,
@@ -340,18 +416,32 @@ public class LDAPDatabase extends ACMEDatabase {
     }
 
     public ACMEOrder getOrder(String orderID) throws Exception {
-        String dn = ATTR_ORDER_ID + "=" + orderID + "," + RDN_ORDER + "," + basedn;
+        String dn = ATTR_ORDER_ID + "=" + orderID + "," + RDN_ORDER + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
         return loadOrder(entry);
     }
 
+    public Collection<ACMEOrder> getOrdersByAccount(String accountID) throws Exception {
+        Collection<ACMEOrder> orders = new ArrayList<>();
+
+        List<LDAPEntry> entries = ldapSearch(
+            RDN_ORDER + "," + baseDN,
+            "(&(" + ATTR_OBJECTCLASS + "=" + OBJ_ORDER +
+                ")(" + ATTR_ACCOUNT_ID + "=" + accountID + "))"
+        );
+        for (LDAPEntry entry : entries) {
+            orders.add(loadOrder(entry));
+        }
+        return orders;
+    }
+
     public Collection<ACMEOrder> getOrdersByAuthorizationAndStatus(String authzID, String status)
             throws Exception {
         Collection<ACMEOrder> orders = new ArrayList<>();
 
         List<LDAPEntry> entries = ldapSearch(
-            RDN_ORDER + "," + basedn,
+            RDN_ORDER + "," + baseDN,
             "(&(" + ATTR_OBJECTCLASS + "=" + OBJ_ORDER +
                 ")(" + ATTR_AUTHORIZATION_ID + "=" + authzID +
                 ")(" + ATTR_STATUS + "=" + status + "))"
@@ -365,7 +455,7 @@ public class LDAPDatabase extends ACMEDatabase {
     public ACMEOrder getOrderByCertificate(String certID)
             throws Exception {
         List<LDAPEntry> entries = ldapSearch(
-            RDN_ORDER + "," + basedn,
+            RDN_ORDER + "," + baseDN,
             "(&(" + ATTR_OBJECTCLASS + "=" + OBJ_ORDER +
                 ")(" + ATTR_CERTIFICATE_ID + "=" + certID + "))"
         );
@@ -377,8 +467,7 @@ public class LDAPDatabase extends ACMEDatabase {
         }
     }
 
-    private static ACMEOrder loadOrder(LDAPEntry entry)
-            throws ParseException {
+    private static ACMEOrder loadOrder(LDAPEntry entry) throws Exception {
         ACMEOrder order = new ACMEOrder();
 
         LDAPAttribute attr = entry.getAttribute(ATTR_ORDER_ID);
@@ -387,6 +476,9 @@ public class LDAPDatabase extends ACMEDatabase {
         attr = entry.getAttribute(ATTR_ACCOUNT_ID);
         order.setAccountID(attr.getStringValues().nextElement());
 
+        attr = entry.getAttribute(ATTR_CREATED);
+        order.setCreationTime(dateFormat.parse(attr.getStringValues().nextElement()));
+
         attr = entry.getAttribute(ATTR_STATUS);
         order.setStatus(attr.getStringValues().nextElement());
 
@@ -406,11 +498,17 @@ public class LDAPDatabase extends ACMEDatabase {
         }
 
         List<ACMEIdentifier> identifiers = new ArrayList<>();
-        for (String type : IDENTIFIER_TYPES) {
-            attr = entry.getAttribute(ATTR_IDENTIFIER + ";" + type);
-            for (String value : Collections.list(attr.getStringValues())) {
-                identifiers.add(new ACMEIdentifier(type, value));
+        attr = entry.getAttribute(ATTR_IDENTIFIER);
+        for (String identifier : attr.getStringValueArray()) {
+            String[] parts = StringUtils.split(identifier, ":", 2);
+            if (parts.length != 2) {
+                throw new Exception("Invalid order identifier: " + identifier);
             }
+
+            String type = parts[0];
+            String value = parts[1];
+
+            identifiers.add(new ACMEIdentifier(type, value));
         }
         order.setIdentifiers(identifiers.toArray(new ACMEIdentifier[0]));
 
@@ -425,6 +523,7 @@ public class LDAPDatabase extends ACMEDatabase {
             new LDAPAttribute(ATTR_OBJECTCLASS, OBJ_ORDER),
             new LDAPAttribute(ATTR_ORDER_ID, order.getID()),
             new LDAPAttribute(ATTR_ACCOUNT_ID, order.getAccountID()),
+            new LDAPAttribute(ATTR_CREATED, dateFormat.format(order.getCreationTime())),
             new LDAPAttribute(ATTR_STATUS, order.getStatus()),
             new LDAPAttribute(ATTR_AUTHORIZATION_ID, order.getAuthzIDs())
         };
@@ -439,11 +538,11 @@ public class LDAPDatabase extends ACMEDatabase {
 
         // identifiers
         ACMEIdentifier[] identifiers = order.getIdentifiers();
-        for (int i = 0; i < identifiers.length; i++) {
+        for (ACMEIdentifier identifier : identifiers) {
             attrSet.add(
                 new LDAPAttribute(
-                    ATTR_IDENTIFIER + ";" + identifiers[i].getType(),
-                    identifiers[i].getValue())
+                    ATTR_IDENTIFIER,
+                    identifier.getType() + ":" + identifier.getValue())
             );
         }
 
@@ -452,13 +551,13 @@ public class LDAPDatabase extends ACMEDatabase {
             attrSet.add(new LDAPAttribute(ATTR_ERROR, error));
         }
 
-        String dn = ATTR_ORDER_ID + "=" + order.getID() + "," + RDN_ORDER + "," + basedn;
+        String dn = ATTR_ORDER_ID + "=" + order.getID() + "," + RDN_ORDER + "," + baseDN;
         LDAPEntry entry = new LDAPEntry(dn, attrSet);
         ldapAdd(entry);
     }
 
     public void updateOrder(ACMEOrder order) throws Exception {
-        String dn = ATTR_ORDER_ID + "=" + order.getID() + "," + RDN_ORDER + "," + basedn;
+        String dn = ATTR_ORDER_ID + "=" + order.getID() + "," + RDN_ORDER + "," + baseDN;
         LDAPModificationSet mods = new LDAPModificationSet();
         mods.add(
             LDAPModification.REPLACE,
@@ -499,7 +598,7 @@ public class LDAPDatabase extends ACMEDatabase {
     public void removeExpiredOrders(Date currentTime) throws Exception {
         String[] attrs = {"1.1"};  // suppress attrs for performance; we only need DN
         List<LDAPEntry> entries = ldapSearch(
-            RDN_ORDER + "," + basedn,
+            RDN_ORDER + "," + baseDN,
             "(" + ATTR_EXPIRES + "<=" + dateFormat.format(currentTime) + ")",
             attrs
         );
@@ -514,7 +613,7 @@ public class LDAPDatabase extends ACMEDatabase {
 
     private ACMEAuthorization getAuthorization(String authzID, LoadChallenges clc)
             throws Exception {
-        String dn = ATTR_AUTHORIZATION_ID + "=" + authzID + "," + RDN_AUTHORIZATION + "," + basedn;
+        String dn = ATTR_AUTHORIZATION_ID + "=" + authzID + "," + RDN_AUTHORIZATION + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
 
@@ -526,6 +625,9 @@ public class LDAPDatabase extends ACMEDatabase {
         attr = entry.getAttribute(ATTR_ACCOUNT_ID);
         authz.setAccountID(attr.getStringValues().nextElement());
 
+        attr = entry.getAttribute(ATTR_CREATED);
+        authz.setCreationTime(dateFormat.parse(attr.getStringValues().nextElement()));
+
         attr = entry.getAttribute(ATTR_EXPIRES);
         if (attr != null) {
             authz.setExpirationTime(dateFormat.parse(attr.getStringValues().nextElement()));
@@ -534,15 +636,19 @@ public class LDAPDatabase extends ACMEDatabase {
         attr = entry.getAttribute(ATTR_STATUS);
         authz.setStatus(attr.getStringValues().nextElement());
 
-        for (String type : IDENTIFIER_TYPES) {
-            attr = entry.getAttribute(ATTR_IDENTIFIER + ";" + type);
-            if (attr != null) {
-                authz.setIdentifier(
-                    new ACMEIdentifier(type, attr.getStringValues().nextElement()));
-                break;
-            }
+        attr = entry.getAttribute(ATTR_IDENTIFIER);
+        String identifier = attr.getStringValues().nextElement();
+
+        String[] parts = StringUtils.split(identifier, ":", 2);
+        if (parts.length != 2) {
+            throw new Exception("Invalid authorization identifier: " + identifier);
         }
 
+        String type = parts[0];
+        String value = parts[1];
+
+        authz.setIdentifier(new ACMEIdentifier(type, value));
+
         attr = entry.getAttribute(ATTR_AUTHORIZATION_WILDCARD);
         if (attr != null) {
             if ("TRUE".equalsIgnoreCase(attr.getStringValues().nextElement())) {
@@ -553,7 +659,7 @@ public class LDAPDatabase extends ACMEDatabase {
         if (clc == LoadChallenges.DoLoad) {
             List<ACMEChallenge> challenges = new ArrayList<>();
             Collection<LDAPEntry> entries = ldapSearch(
-                RDN_CHALLENGE + "," + basedn,
+                RDN_CHALLENGE + "," + baseDN,
                 "(&(" + ATTR_OBJECTCLASS + "=" + OBJ_CHALLENGE +
                     ")(" + ATTR_AUTHORIZATION_ID + "=" + authzID + "))"
             );
@@ -569,7 +675,7 @@ public class LDAPDatabase extends ACMEDatabase {
     private ACMEChallenge getChallenge(String challengeID)
             throws Exception {
         String dn = ATTR_CHALLENGE_ID + "=" + challengeID
-                        + "," + RDN_CHALLENGE + "," + basedn;
+                        + "," + RDN_CHALLENGE + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
         return loadChallenge(entry);
@@ -637,8 +743,9 @@ public class LDAPDatabase extends ACMEDatabase {
             new LDAPAttribute(ATTR_OBJECTCLASS, OBJ_AUTHORIZATION),
             new LDAPAttribute(ATTR_AUTHORIZATION_ID, authorization.getID()),
             new LDAPAttribute(ATTR_ACCOUNT_ID, authorization.getAccountID()),
+            new LDAPAttribute(ATTR_CREATED, dateFormat.format(authorization.getCreationTime())),
             new LDAPAttribute(ATTR_STATUS, authorization.getStatus()),
-            new LDAPAttribute(ATTR_IDENTIFIER + ";" + identifier.getType(), identifier.getValue())
+            new LDAPAttribute(ATTR_IDENTIFIER, identifier.getType() + ":" + identifier.getValue())
         };
         LDAPAttributeSet attrSet = new LDAPAttributeSet(attrs);
 
@@ -652,7 +759,7 @@ public class LDAPDatabase extends ACMEDatabase {
         attrSet.add(new LDAPAttribute(ATTR_AUTHORIZATION_WILDCARD, wildcardValue));
 
         String dn = ATTR_AUTHORIZATION_ID + "=" + authorization.getID()
-                        + "," + RDN_AUTHORIZATION + "," + basedn;
+                        + "," + RDN_AUTHORIZATION + "," + baseDN;
         LDAPEntry entry = new LDAPEntry(dn, attrSet);
         ldapAdd(entry);
     }
@@ -681,14 +788,14 @@ public class LDAPDatabase extends ACMEDatabase {
         LDAPAttributeSet attrSet = new LDAPAttributeSet(attrs);
 
         String dn = ATTR_CHALLENGE_ID + "=" + challenge.getID()
-                        + "," + RDN_CHALLENGE + "," + basedn;
+                        + "," + RDN_CHALLENGE + "," + baseDN;
         LDAPEntry entry = new LDAPEntry(dn, attrSet);
         ldapAdd(entry);
     }
 
     public void updateAuthorization(ACMEAuthorization authorization) throws Exception {
         String dn = ATTR_AUTHORIZATION_ID + "=" + authorization.getID()
-                        + "," + RDN_AUTHORIZATION + "," + basedn;
+                        + "," + RDN_AUTHORIZATION + "," + baseDN;
         LDAPModificationSet mods = new LDAPModificationSet();
         mods.add(
             LDAPModification.REPLACE,
@@ -715,13 +822,13 @@ public class LDAPDatabase extends ACMEDatabase {
             getAuthorization(authorization.getID(), LoadChallenges.DoLoad);
         for (ACMEChallenge challenge : authzORIG.getChallenges()) {
             dn = ATTR_CHALLENGE_ID + "=" + challenge.getID()
-                    + "," + RDN_CHALLENGE + "," + basedn;
+                    + "," + RDN_CHALLENGE + "," + baseDN;
             ldapDelete(dn, OnNoSuchObject.Ignore);
         }
         // now add the possibly-changed challenges.
         for (ACMEChallenge challenge : authorization.getChallenges()) {
             dn = ATTR_CHALLENGE_ID + "=" + challenge.getID()
-                    + "," + RDN_CHALLENGE + "," + basedn;
+                    + "," + RDN_CHALLENGE + "," + baseDN;
             addChallenge(authorization.getAccountID(), challenge);
         }
         // NOTE: there are optimisation opportunities here.
@@ -771,12 +878,12 @@ public class LDAPDatabase extends ACMEDatabase {
          *   whether the identifier began with "*." or not.
          */
         List<LDAPEntry> entries = ldapSearch(
-            RDN_AUTHORIZATION + "," + basedn,
+            RDN_AUTHORIZATION + "," + baseDN,
             "(&(" + ATTR_OBJECTCLASS + "=" + OBJ_AUTHORIZATION
                 + ")(" + ATTR_ACCOUNT_ID + "=" + accountID
                 + ")(!(" + ATTR_EXPIRES + "<=" + dateFormat.format(time) + ")"
                 + ")(" + ATTR_STATUS + "=valid"
-                + ")(" + ATTR_IDENTIFIER + ";" + identifier.getType() + "=" + ident
+                + ")(" + ATTR_IDENTIFIER + "=" + identifier.getType() + ":" + ident
                 + ")(" + ATTR_AUTHORIZATION_WILDCARD + "=" + (wildcard ? "TRUE" : "FALSE")
                 + "))"
         );
@@ -786,7 +893,7 @@ public class LDAPDatabase extends ACMEDatabase {
     public void removeExpiredAuthorizations(Date currentTime) throws Exception {
         String[] attrs = {"1.1"};  // suppress attrs for performance; we only need DN
         List<LDAPEntry> entries = ldapSearch(
-            RDN_AUTHORIZATION + "," + basedn,
+            RDN_AUTHORIZATION + "," + baseDN,
             "(" + ATTR_EXPIRES + "<=" + dateFormat.format(currentTime) + ")",
             attrs
         );
@@ -796,14 +903,17 @@ public class LDAPDatabase extends ACMEDatabase {
     }
 
     public ACMECertificate getCertificate(String certID) throws Exception {
-        String dn = ATTR_CERTIFICATE_ID + "=" + certID + "," + RDN_CERTIFICATE + "," + basedn;
+        String dn = ATTR_CERTIFICATE_ID + "=" + certID + "," + RDN_CERTIFICATE + "," + baseDN;
         LDAPEntry entry = ldapGet(dn);
         if (entry == null) return null;
 
         ACMECertificate certificate = new ACMECertificate();
         certificate.setID(certID);
 
-        LDAPAttribute attr = entry.getAttribute(ATTR_USER_CERTIFICATE);
+        LDAPAttribute attr = entry.getAttribute(ATTR_CREATED);
+        certificate.setCreationTime(dateFormat.parse(attr.getStringValues().nextElement()));
+
+        attr = entry.getAttribute(ATTR_USER_CERTIFICATE);
         certificate.setData(attr.getByteValueArray()[0]);
 
         attr = entry.getAttribute(ATTR_EXPIRES);
@@ -816,10 +926,11 @@ public class LDAPDatabase extends ACMEDatabase {
 
     public void addCertificate(String certID, ACMECertificate certificate) throws Exception {
 
-        String dn = ATTR_CERTIFICATE_ID + "=" + certID + "," + RDN_CERTIFICATE + "," + basedn;
+        String dn = ATTR_CERTIFICATE_ID + "=" + certID + "," + RDN_CERTIFICATE + "," + baseDN;
         LDAPAttribute[] attrs = {
                 new LDAPAttribute(ATTR_OBJECTCLASS, OBJ_CERTIFICATE),
                 new LDAPAttribute(ATTR_CERTIFICATE_ID, certID),
+                new LDAPAttribute(ATTR_CREATED, dateFormat.format(certificate.getCreationTime())),
                 new LDAPAttribute(ATTR_USER_CERTIFICATE, certificate.getData())
         };
         LDAPAttributeSet attrSet = new LDAPAttributeSet(attrs);
@@ -838,7 +949,7 @@ public class LDAPDatabase extends ACMEDatabase {
     public void removeExpiredCertificates(Date currentTime) throws Exception {
         String[] attrs = {"1.1"};  // suppress attrs for performance; we only need DN
         List<LDAPEntry> entries = ldapSearch(
-            RDN_CERTIFICATE + "," + basedn,
+            RDN_CERTIFICATE + "," + baseDN,
             "(" + ATTR_EXPIRES + "<=" + dateFormat.format(currentTime) + ")",
             attrs
         );
@@ -847,7 +958,6 @@ public class LDAPDatabase extends ACMEDatabase {
         }
     }
 
-
     /* LOW LEVEL LDAP METHODS */
 
     void ldapAdd(LDAPEntry entry) throws Exception {
@@ -957,4 +1067,10 @@ public class LDAPDatabase extends ACMEDatabase {
 
         return l;
     }
+
+    public void close() throws Exception {
+        if (monitor != null) {
+            monitor.stop();
+        }
+    }
 }
