@@ -87,6 +87,10 @@ class CertCLI(pki.cli.CLI):
         if not_after:
             print('  Not Valid After: %s' % CertCLI.convert_millis_to_date(not_after))
 
+        trust_flags = cert.get('trust_flags')
+        if trust_flags:
+            print('  NSSDB Trust flag: %s' % trust_flags)
+
         if show_all:
             print('  Certificate: %s' % cert['data'])
             print('  Request: %s' % cert['request'])
@@ -150,7 +154,7 @@ class CertFindCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -159,7 +163,7 @@ class CertFindCLI(pki.cli.CLI):
         first = True
         results = []
 
-        for subsystem in instance.subsystems:
+        for subsystem in instance.get_subsystems():
 
             # Retrieve the subsystem's system certificate
             certs = subsystem.find_system_certs()
@@ -250,7 +254,7 @@ class CertShowCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -260,7 +264,7 @@ class CertShowCLI(pki.cli.CLI):
 
         # If cert ID is instance specific, get it from first subsystem
         if not subsystem_name:
-            subsystem_name = instance.subsystems[0].name
+            subsystem_name = instance.get_subsystems()[0].name
 
         subsystem = instance.get_subsystem(subsystem_name)
 
@@ -274,17 +278,14 @@ class CertShowCLI(pki.cli.CLI):
         CertCLI.print_system_cert(cert, show_all)
 
         if pretty_print:
+
+            print()
+
             nssdb = instance.open_nssdb()
             try:
-                output = nssdb.get_cert(
+                nssdb.show_cert(
                     nickname=cert['nickname'],
-                    token=cert['token'],
-                    output_format='pretty-print',
-                    output_text=True)
-
-                print()
-                print(output)
-
+                    token=cert['token'])
             finally:
                 nssdb.close()
 
@@ -345,7 +346,7 @@ class CertUpdateCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -355,7 +356,7 @@ class CertUpdateCLI(pki.cli.CLI):
 
         # If cert ID is instance specific, get it from first subsystem
         if not subsystem_name:
-            subsystem_name = instance.subsystems[0].name
+            subsystem_name = instance.get_subsystems()[0].name
 
         subsystem = instance.get_subsystem(subsystem_name)
 
@@ -574,7 +575,7 @@ class CertCreateCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -659,7 +660,7 @@ class CertImportCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -816,7 +817,7 @@ class CertExportCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -826,7 +827,7 @@ class CertExportCLI(pki.cli.CLI):
 
         # If cert ID is instance specific, get it from first subsystem
         if not subsystem_name:
-            subsystem_name = instance.subsystems[0].name
+            subsystem_name = instance.get_subsystems()[0].name
 
         subsystem = instance.get_subsystem(subsystem_name)
 
@@ -844,7 +845,6 @@ class CertExportCLI(pki.cli.CLI):
             sys.exit(1)
 
         if cert_id == 'sslserver':
-            # get nickname and token from serverCertNick.conf
             full_name = instance.get_sslserver_cert_nickname()
             i = full_name.find(':')
             if i < 0:
@@ -870,7 +870,7 @@ class CertExportCLI(pki.cli.CLI):
 
                 logger.info('Exporting %s certificate into %s.', cert_id, cert_file)
 
-                cert_data = cert.get('data', None)
+                cert_data = cert.get('data')
                 if cert_data is None:
                     logger.error('Unable to find certificate data for %s', cert_id)
                     sys.exit(1)
@@ -883,7 +883,7 @@ class CertExportCLI(pki.cli.CLI):
 
                 logger.info('Exporting %s CSR into %s.', cert_id, csr_file)
 
-                cert_request = cert.get('request', None)
+                cert_request = cert.get('request')
                 if cert_request is None:
                     logger.error('Unable to find certificate request for %s', cert_id)
                     sys.exit(1)
@@ -982,7 +982,7 @@ class CertRemoveCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -1106,7 +1106,7 @@ class CertFixCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -1123,7 +1123,7 @@ class CertFixCLI(pki.cli.CLI):
         # 1. Make a list of certs to fix OR use the list provided through CLI options
         if all_certs:
             # TODO: Identify only certs that are EXPIRED or ALMOST EXPIRED
-            for subsystem in instance.subsystems:
+            for subsystem in instance.get_subsystems():
                 # Retrieve the subsystem's system certificate
                 certs = subsystem.find_system_certs()
 
@@ -1169,7 +1169,7 @@ class CertFixCLI(pki.cli.CLI):
             if 'sslserver' in fix_certs or 'subsystem' in fix_certs:
                 # If the cert is either sslserver/subsystem, disable selftest for all
                 # subsystems since all subsystems use these 2 certs.
-                target_subsys = set(instance.subsystems)
+                target_subsys = set(instance.get_subsystems())
 
             else:
                 for cert_id in fix_certs:
@@ -1232,8 +1232,9 @@ class CertFixCLI(pki.cli.CLI):
                     for cert_id in fix_certs:
                         logger.info('Requesting new cert for %s', cert_id)
                         instance.cert_create(
-                            cert_id=cert_id, renew=True,
-                            username=agent_uid, password=agent_pass, secure_port=port)
+                            cert_id=cert_id, renew=True, username=agent_uid,
+                            password=agent_pass, secure_port=port,
+                            client_nssdb=instance.nssdb_dir)
                     for serial in extra_certs:
                         output = instance.cert_file('{}-renewed'.format(serial))
                         logger.info(
@@ -1241,8 +1242,9 @@ class CertFixCLI(pki.cli.CLI):
                             serial, output)
                         try:
                             instance.cert_create(
-                                serial=serial, renew=True, output=output,
-                                username=agent_uid, password=agent_pass, secure_port=port)
+                                serial=serial, renew=True, output=output, username=agent_uid,
+                                password=agent_pass, secure_port=port,
+                                client_nssdb=instance.nssdb_dir)
                         except pki.PKIException as e:
                             logger.error("Failed to renew certificate %s: %s", serial, e)
 
