@@ -19,6 +19,7 @@
 #
 
 from __future__ import absolute_import
+import base64
 import json
 import ldap
 import logging
@@ -303,7 +304,7 @@ class PKIDeployer:
 
         # Sleep for a bit to allow the install token to replicate to other clones.
         # In the future this can be replaced with signed tokens.
-        # https://pagure.io/dogtagpki/issue/2831
+        # https://github.com/dogtagpki/pki/issues/2951
         #
         # The default sleep time is 5s.
 
@@ -340,23 +341,18 @@ class PKIDeployer:
         request.tag = tag
         self.config_client.set_system_cert_info(request, tag)
 
-        if tag != 'sslserver' and request.clone:
-            logger.info('%s certificate is already set up', tag)
-            return None
-
-        system_list = self.instance.tomcat_instance_subsystems()
-
-        if tag == 'sslserver' and len(system_list) > 1:
-            logger.info('sslserver certificate is already set up')
-            return None
-
-        if tag == 'subsystem' and len(system_list) > 1:
-            logger.info('subsystem certificate is already set up')
-            return None
-
         return client.setupCert(request)
 
-    def setup_admin(self, client):
+    def setup_admin(self, subsystem, client):
+
+        uid = self.mdict['pki_admin_uid']
+        full_name = self.mdict['pki_admin_name']
+        email = self.mdict['pki_admin_email']
+        password = self.mdict['pki_admin_password']
+
+        tps_profiles = None
+        if subsystem.type == 'TPS':
+            tps_profiles = ['All Profiles']
 
         request = pki.system.AdminSetupRequest()
         request.pin = self.mdict['pki_one_time_pin']
@@ -366,11 +362,43 @@ class PKIDeployer:
 
         response = client.setupAdmin(request)
 
+        subsystem.add_user(
+            uid,
+            full_name=full_name,
+            email=email,
+            password=password,
+            user_type='adminType',
+            state='1',
+            tps_profiles=tps_profiles)
+
+        admin_groups = subsystem.config['preop.admin.group']
+        groups = [x.strip() for x in admin_groups.split(',')]
+
+        if subsystem.config['securitydomain.select'] == 'new':
+            groups.extend([
+                'Security Domain Administrators',
+                'Enterprise CA Administrators',
+                'Enterprise KRA Administrators',
+                'Enterprise RA Administrators',
+                'Enterprise TKS Administrators',
+                'Enterprise OCSP Administrators',
+                'Enterprise TPS Administrators'
+            ])
+
+        for group in groups:
+            logger.info('Adding %s into %s', uid, group)
+            subsystem.add_group_member(group, uid)
+
+        admin_cert = response['adminCert']['cert']
+        cert_data = base64.b64decode(admin_cert)
+
+        logger.info('Adding certificate for %s', uid)
+        subsystem.add_user_cert(uid, cert_data=cert_data, cert_format='DER')
+
         if config.str2bool(self.mdict['pki_external']) \
                 or config.str2bool(self.mdict['pki_standalone']) \
                 or not config.str2bool(self.mdict['pki_import_admin_cert']):
 
-            admin_cert = response['adminCert']['cert']
             self.config_client.process_admin_cert(admin_cert)
 
     def backup_keys(self, instance, subsystem):
@@ -396,6 +424,65 @@ class PKIDeployer:
         finally:
             shutil.rmtree(tmpdir)
 
+    def setup_database_user(self, instance, subsystem):
+
+        logger.info('Adding pkidbuser')
+        subsystem.add_user(
+            'pkidbuser',
+            full_name='pkidbuser',
+            user_type='agentType',
+            state='1')
+
+        subsystem_cert = subsystem.get_subsystem_cert('subsystem')
+        subject = subsystem_cert['subject']
+
+        nssdb = instance.open_nssdb()
+        try:
+            cert_data = nssdb.get_cert(
+                nickname=subsystem_cert['nickname'],
+                token=subsystem_cert['token'])
+        finally:
+            nssdb.close()
+
+        logger.info('Adding subsystem cert into pkidbuser')
+        subsystem.add_user_cert('pkidbuser', cert_data=cert_data, cert_format='PEM')
+
+        logger.info('Linking pkidbuser to subsystem cert')
+        subsystem.modify_user('pkidbuser', add_see_also=subject)
+
+        logger.info('Finding other users linked to subsystem cert')
+        users = subsystem.find_users(see_also=subject)
+
+        for user in users['entries']:
+            uid = user['id']
+
+            if uid == 'pkidbuser':
+                continue
+
+            logger.info('Unlinking %s from subsystem cert ', uid)
+            subsystem.modify_user(uid, del_see_also=subject)
+
+        # workaround for https://github.com/dogtagpki/pki/issues/2154
+
+        if subsystem.type == 'CA':
+            groups = ['Subsystem Group', 'Certificate Manager Agents']
+
+        elif subsystem.type == 'KRA':
+            groups = ['Data Recovery Manager Agents', 'Trusted Managers']
+
+        elif subsystem.type == 'OCSP':
+            groups = ['Trusted Managers']
+
+        elif subsystem.type == 'TKS':
+            groups = ['Token Key Service Manager Agents']
+
+        else:
+            groups = []
+
+        for group in groups:
+            logger.info('Adding pkidbuser into %s', group)
+            subsystem.add_group_member(group, 'pkidbuser')
+
     def get_tps_connector(self, instance, subsystem):
 
         tks_uri = self.mdict['pki_tks_uri']
@@ -550,12 +637,7 @@ class PKIDeployer:
         else:
             return None
 
-    def import_shared_secret(self, instance, subsystem, shared_secret):
-
-        hostname = self.mdict['pki_hostname']
-
-        server_config = instance.get_server_config()
-        securePort = server_config.get_secure_port()
+    def import_shared_secret(self, instance, subsystem, secret_nickname, shared_secret):
 
         subsystem_cert = subsystem.get_subsystem_cert('subsystem')
 
@@ -565,8 +647,6 @@ class PKIDeployer:
         if pki.nssdb.normalize_token(token):
             nickname = token + ':' + nickname
 
-        secret_nickname = 'TPS-%s-%s sharedSecret' % (hostname, securePort)
-
         cmd = [
             'pki',
             '-d', instance.nssdb_dir,
@@ -592,6 +672,13 @@ class PKIDeployer:
         # TODO: Clean up the code and determine whether TKS and TPS are in the same
         # instance automatically.
 
+        hostname = self.mdict['pki_hostname']
+
+        server_config = instance.get_server_config()
+        securePort = server_config.get_secure_port()
+
+        secret_nickname = 'TPS-%s-%s sharedSecret' % (hostname, securePort)
+
         logger.info('Searching for TPS connector in TKS')
         tps_connector = self.get_tps_connector(instance, subsystem)
 
@@ -618,4 +705,26 @@ class PKIDeployer:
 
         if config.str2bool(self.mdict['pki_import_shared_secret']):
             logger.info('Importing shared secret')
-            self.import_shared_secret(instance, subsystem, shared_secret)
+            self.import_shared_secret(instance, subsystem, secret_nickname, shared_secret)
+
+        subsystem.config['conn.tks1.tksSharedSymKeyName'] = secret_nickname
+        subsystem.save()
+
+    def finalize_subsystem(self, subsystem):
+
+        if subsystem.type != 'CA':
+            ca_type = subsystem.config.get('preop.ca.type')
+            if ca_type:
+                subsystem.config['cloning.ca.type'] = ca_type
+
+        # save EC type for sslserver cert (if present)
+        ec_type = subsystem.config.get('preop.cert.sslserver.ec.type', 'ECDHE')
+        subsystem.config['jss.ssl.sslserver.ectype'] = ec_type
+
+        for key in list(subsystem.config.keys()):
+            if key.startswith('preop.'):
+                del subsystem.config[key]
+
+        subsystem.config['cs.state'] = '1'
+
+        subsystem.save()
