@@ -26,6 +26,7 @@ import getpass
 import ldap
 import logging
 import os
+import string
 import xml.etree.ElementTree as ET
 
 from six.moves import input  # pylint: disable=W0622,F0401
@@ -271,6 +272,7 @@ class PKIConfigParser:
         application_version = str(pki.util.Version(
             pki.specification_version()))
 
+        charset = string.digits + string.ascii_lowercase + string.ascii_uppercase
         self.deployer.main_config = configparser.SafeConfigParser({
             'application_version': application_version,
             'pki_instance_name': default_instance_name,
@@ -283,7 +285,7 @@ class PKIConfigParser:
             'java_home': java_home,
             'home_dir': os.path.expanduser("~"),
             'pki_hostname': self.deployer.hostname,
-            'pki_random_ajp_secret': pki.generate_password()})
+            'pki_random_ajp_secret': pki.generate_password(charset, length=25)})
 
         # Make keys case-sensitive!
         self.deployer.main_config.optionxform = str
@@ -606,9 +608,6 @@ class PKIConfigParser:
             if 'pki_subordinate' not in self.mdict or\
                not len(self.mdict['pki_subordinate']):
                 self.mdict['pki_subordinate'] = "false"
-            if 'pki_san_inject' not in self.mdict or\
-               not len(self.mdict['pki_san_inject']):
-                self.mdict['pki_san_inject'] = "false"
 
             # PKI Target (slot substitution) name/value pairs
             self.mdict['pki_target_cs_cfg'] = \
@@ -854,89 +853,6 @@ class PKIConfigParser:
                 ".pid"
             self.mdict['TOMCAT_SERVER_PORT_SLOT'] = \
                 self.mdict['pki_tomcat_server_port']
-            self.mdict['TOMCAT_SSL_VERSION_RANGE_STREAM_SLOT'] = \
-                "tls1_1:tls1_2"
-            self.mdict['TOMCAT_SSL_VERSION_RANGE_DATAGRAM_SLOT'] = \
-                "tls1_1:tls1_2"
-            ##
-            # Reminder: if the following cipher lists are updated, be sure
-            # to remember to update pki/base/server/share/conf/ciphers.info
-            # accordingly
-            #
-            if self.mdict['pki_sslserver_key_type'] == "ecc":
-                self.mdict['TOMCAT_SSL_RANGE_CIPHERS_SLOT'] = \
-                    "-TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_DHE_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_DHE_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_DHE_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "-TLS_DHE_RSA_WITH_AES_256_CBC_SHA256," + \
-                    "-TLS_DHE_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "-TLS_RSA_WITH_AES_256_CBC_SHA256," + \
-                    "-TLS_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "-TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384," + \
-                    "+TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384"
-            else:
-                self.mdict['TOMCAT_SSL_RANGE_CIPHERS_SLOT'] = \
-                    "-TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDH_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_DHE_DSS_WITH_AES_256_CBC_SHA," + \
-                    "-TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "+TLS_DHE_RSA_WITH_AES_128_CBC_SHA," + \
-                    "+TLS_DHE_RSA_WITH_AES_256_CBC_SHA," + \
-                    "+TLS_DHE_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "+TLS_DHE_RSA_WITH_AES_256_CBC_SHA256," + \
-                    "+TLS_DHE_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_DHE_DSS_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "-TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_RSA_WITH_AES_128_CBC_SHA256," + \
-                    "-TLS_RSA_WITH_AES_256_CBC_SHA256," + \
-                    "-TLS_RSA_WITH_AES_128_GCM_SHA256," + \
-                    "-TLS_RSA_WITH_3DES_EDE_CBC_SHA," + \
-                    "-TLS_RSA_WITH_AES_128_CBC_SHA," + \
-                    "-TLS_RSA_WITH_AES_256_CBC_SHA," + \
-                    "+TLS_DHE_RSA_WITH_AES_256_GCM_SHA384," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384," + \
-                    "+TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384," + \
-                    "-TLS_RSA_WITH_AES_256_GCM_SHA384"
 
             # tps parameters
             self.mdict['TOKENDB_HOST_SLOT'] = \
