@@ -879,6 +879,10 @@ class PKISubsystem(object):
             create_base=False,
             create_containers=False,
             rebuild_indexes=False,
+            setup_replication=False,
+            replication_security=None,
+            replication_port=None,
+            master_replication_port=None,
             setup_db_manager=False,
             setup_vlv_indexes=False,
             as_current_user=False):
@@ -900,6 +904,18 @@ class PKISubsystem(object):
         if rebuild_indexes:
             cmd.append('--rebuild-indexes')
 
+        if setup_replication:
+            cmd.append('--setup-replication')
+
+        if replication_security:
+            cmd.extend(['--replication-security', replication_security])
+
+        if replication_port:
+            cmd.extend(['--replication-port', replication_port])
+
+        if master_replication_port:
+            cmd.extend(['--master-replication-port', master_replication_port])
+
         if setup_db_manager:
             cmd.append('--setup-db-manager')
 
@@ -944,7 +960,7 @@ class PKISubsystem(object):
 
         self.run(cmd, as_current_user=as_current_user)
 
-    def request_range(self, master_url, range_type, install_token):
+    def request_range(self, master_url, install_token, range_type):
 
         cmd = [
             'pki',
@@ -968,23 +984,23 @@ class PKISubsystem(object):
 
         return json.loads(output.decode())
 
-    def update_ranges(self, master_url, install_token):
+    def request_ranges(self, master_url, install_token):
 
-        logger.info('Updating request ID range')
+        logger.info('Requesting request ID range')
 
-        request_range = self.request_range(master_url, 'request', install_token)
+        request_range = self.request_range(master_url, install_token, 'request')
         self.config['dbs.beginRequestNumber'] = request_range['begin']
         self.config['dbs.endRequestNumber'] = request_range['end']
 
-        logger.info('Updating serial number range')
+        logger.info('Requesting serial number range')
 
-        serial_range = self.request_range(master_url, 'serialNo', install_token)
+        serial_range = self.request_range(master_url, install_token, 'serialNo')
         self.config['dbs.beginSerialNumber'] = serial_range['begin']
         self.config['dbs.endSerialNumber'] = serial_range['end']
 
-        logger.info('Updating replica ID range')
+        logger.info('Requesting replica ID range')
 
-        replica_range = self.request_range(master_url, 'replicaId', install_token)
+        replica_range = self.request_range(master_url, install_token, 'replicaId')
         self.config['dbs.beginReplicaNumber'] = replica_range['begin']
         self.config['dbs.endReplicaNumber'] = replica_range['end']
 
@@ -992,6 +1008,18 @@ class PKISubsystem(object):
 
         self.save()
 
+    def update_ranges(self, as_current_user=False):
+
+        cmd = [self.name + '-range-update']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
     def retrieve_config(self, master_url, names, substores, install_token):
 
         cmd = [
@@ -1077,7 +1105,311 @@ class PKISubsystem(object):
                 master_port == replica_port:
             raise Exception('Master and replica must not share LDAP database')
 
-    def run(self, args, as_current_user=False):
+    def create_security_domain(self, as_current_user=False):
+
+        cmd = ['sd-create']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def add_security_domain_host(
+            self,
+            host_id,
+            hostname,
+            unsecure_port='8080',
+            secure_port='8443',
+            domain_manager=False,
+            clone=False,
+            as_current_user=False):
+
+        cmd = [
+            'sd-host-add',
+            '--hostname', hostname
+        ]
+
+        if unsecure_port:
+            cmd.append('--unsecure-port')
+            cmd.append(unsecure_port)
+
+        if secure_port:
+            cmd.append('--secure-port')
+            cmd.append(secure_port)
+
+        if domain_manager:
+            cmd.append('--domain-manager')
+
+        if clone:
+            cmd.append('--clone')
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(host_id)
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def join_security_domain(
+            self,
+            install_token,
+            host_id,
+            hostname,
+            unsecure_port='8080',
+            secure_port='8443',
+            domain_manager=False,
+            clone=False):
+
+        sd_hostname = self.config['securitydomain.host']
+        sd_port = self.config['securitydomain.httpsadminport']
+        sd_url = 'https://%s:%s' % (sd_hostname, sd_port)
+
+        cmd = [
+            'pki',
+            '-d', self.instance.nssdb_dir,
+            '-f', self.instance.password_conf,
+            '-U', sd_url,
+            'securitydomain-join',
+            '--session', install_token.token,
+            '--type', self.type,
+            '--hostname', hostname,
+            '--unsecure-port', unsecure_port,
+            '--secure-port', secure_port
+        ]
+
+        if domain_manager:
+            cmd.append('--domain-manager')
+
+        if clone:
+            cmd.append('--clone')
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(host_id)
+
+        logger.debug('Command: %s', ' '.join(cmd))
+        subprocess.check_call(cmd)
+
+    def find_groups(self, as_current_user=False):
+
+        cmd = [self.name + '-group-find']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def find_group_members(self, group_id, as_current_user=False):
+
+        cmd = [self.name + '-group-member-find']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append('--output-format')
+        cmd.append('json')
+
+        cmd.append(group_id)
+
+        result = self.run(
+            cmd,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+        return json.loads(result.stdout.decode())
+
+    def add_group_member(self, group_id, member_id, as_current_user=False):
+
+        cmd = [self.name + '-group-member-add']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(group_id)
+        cmd.append(member_id)
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def find_users(self, see_also=None, as_current_user=False):
+
+        cmd = [self.name + '-user-find']
+
+        if see_also:
+            cmd.append('--see-also')
+            cmd.append(see_also)
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append('--output-format')
+        cmd.append('json')
+
+        result = self.run(
+            cmd,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+        return json.loads(result.stdout.decode())
+
+    def get_user(self, user_id, as_current_user=False):
+
+        cmd = [self.name + '-user-show']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append('--output-format')
+        cmd.append('json')
+
+        cmd.append(user_id)
+
+        result = self.run(
+            cmd,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+        return json.loads(result.stdout.decode())
+
+    def add_user(self,
+                 user_id,
+                 full_name=None,
+                 email=None,
+                 password=None,
+                 phone=None,
+                 user_type=None,
+                 state=None,
+                 tps_profiles=None,
+                 as_current_user=False):
+
+        cmd = [self.name + '-user-add']
+
+        if full_name:
+            cmd.append('--full-name')
+            cmd.append(full_name)
+
+        if email:
+            cmd.append('--email')
+            cmd.append(email)
+
+        if password:
+            cmd.append('--password')
+            cmd.append(password)
+
+        if phone:
+            cmd.append('--phone')
+            cmd.append(phone)
+
+        if user_type:
+            cmd.append('--type')
+            cmd.append(user_type)
+
+        if state:
+            cmd.append('--state')
+            cmd.append(state)
+
+        if tps_profiles:
+            cmd.append('--tps-profiles')
+            cmd.append(','.join(tps_profiles))
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(user_id)
+
+        self.run(
+            cmd,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+    def modify_user(self, user_id, add_see_also=None, del_see_also=None,
+                    as_current_user=False):
+
+        cmd = [self.name + '-user-mod']
+
+        if add_see_also:
+            cmd.append('--add-see-also')
+            cmd.append(add_see_also)
+
+        if del_see_also:
+            cmd.append('--del-see-also')
+            cmd.append(del_see_also)
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(user_id)
+
+        self.run(
+            cmd,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+    def add_user_cert(self, user_id,
+                      cert_data=None,
+                      cert_path=None,
+                      cert_format='PEM',
+                      as_current_user=False):
+
+        cmd = [self.name + '-user-cert-add']
+
+        if cert_path:
+            cmd.append('--cert')
+            cmd.append(cert_path)
+
+        if cert_format:
+            cmd.append('--format')
+            cmd.append(cert_format)
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(user_id)
+
+        self.run(
+            cmd,
+            input=cert_data,
+            as_current_user=as_current_user,
+            capture_output=True)
+
+    def run(self,
+            args,
+            input=None,  # pylint: disable=W0622
+            as_current_user=False,
+            capture_output=False):
 
         java_path = os.getenv('PKI_JAVA_PATH')
         java_home = self.instance.config['JAVA_HOME']
@@ -1100,7 +1432,7 @@ class PKISubsystem(object):
             # switch to systemd user if different from current user
             username = pwd.getpwuid(os.getuid()).pw_name
             if username != self.instance.user:
-                cmd.extend(['sudo', '-u', self.instance.user])
+                cmd.extend(['/usr/sbin/runuser', '-u', self.instance.user, '--'])
 
         if os.path.exists(java_path):
             cmd.extend([java_path])
@@ -1134,7 +1466,11 @@ class PKISubsystem(object):
         logger.debug('Command: %s', ' '.join(cmd))
 
         try:
-            subprocess.run(cmd, check=True)
+            return subprocess.run(
+                cmd,
+                input=input,
+                capture_output=capture_output,
+                check=True)
 
         except KeyboardInterrupt:
             logger.debug('Server stopped')
@@ -1156,6 +1492,38 @@ class CASubsystem(PKISubsystem):
 
         return profile_configs
 
+    def import_profiles(
+            self,
+            input_folder=None,
+            as_current_user=False):
+
+        cmd = [self.name + '-profile-import']
+
+        if input_folder:
+            cmd.extend(['--input-folder', input_folder])
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def remove_cert(self, serial_number, as_current_user=False):
+
+        cmd = ['ca-cert-del']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        cmd.append(serial_number)
+
+        self.run(cmd, as_current_user=as_current_user)
+
     def find_cert_requests(self, cert=None):
 
         base_dn = self.config['internaldb.basedn']
@@ -1213,6 +1581,40 @@ class CASubsystem(PKISubsystem):
 
         return request
 
+    def enable_subsystem(self, subsystem_id):
+
+        pattern = re.compile(r'subsystem\.(.*)\.id')
+
+        for key in list(self.config.keys()):
+
+            m = pattern.match(key)
+            if not m:
+                continue
+
+            value = self.config[key]
+            if value != subsystem_id:
+                continue
+
+            subsystem_number = m.group(1)
+            self.config['subsystem.%s.enabled' % subsystem_number] = 'true'
+
+    def disable_subsystem(self, subsystem_id):
+
+        pattern = re.compile(r'subsystem\.(.*)\.id')
+
+        for key in list(self.config.keys()):
+
+            m = pattern.match(key)
+            if not m:
+                continue
+
+            value = self.config[key]
+            if value != subsystem_id:
+                continue
+
+            subsystem_number = m.group(1)
+            self.config['subsystem.%s.enabled' % subsystem_number] = 'false'
+
 
 class KRASubsystem(PKISubsystem):
 
