@@ -17,18 +17,21 @@ import org.dogtagpki.cli.CommandCLI;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.netscape.certsrv.base.IConfigStore;
 import com.netscape.cms.servlet.csadmin.LDAPConfigurator;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmscore.base.ConfigStorage;
 import com.netscape.cmscore.base.FileConfigStore;
+import com.netscape.cmscore.ldapconn.LDAPAuthenticationConfig;
 import com.netscape.cmscore.ldapconn.LDAPConfig;
+import com.netscape.cmscore.ldapconn.LDAPConnectionConfig;
 import com.netscape.cmscore.ldapconn.LdapAuthInfo;
 import com.netscape.cmscore.ldapconn.LdapBoundConnection;
 import com.netscape.cmscore.ldapconn.LdapConnInfo;
+import com.netscape.cmscore.ldapconn.PKISocketConfig;
 import com.netscape.cmscore.ldapconn.PKISocketFactory;
 import com.netscape.cmsutil.password.IPasswordStore;
+import com.netscape.cmsutil.password.PasswordStoreConfig;
 
 /**
  * @author Endi S. Dewata
@@ -96,16 +99,12 @@ public class SubsystemDBRemoveCLI extends CommandCLI {
         }
 
         String instanceId = cs.getInstanceID();
-        String pwdClass = cs.getString("passwordClass");
-        String pwdPath = cs.getString("passwordFile", null);
 
-        logger.info("Creating " + pwdClass);
-        IPasswordStore passwordStore = (IPasswordStore) Class.forName(pwdClass).newInstance();
-        passwordStore.init(pwdPath);
-        passwordStore.setId(instanceId);
+        PasswordStoreConfig psc = cs.getPasswordStoreConfig();
+        IPasswordStore passwordStore = IPasswordStore.create(psc);
 
-        IConfigStore connConfig = ldapConfig.getSubStore("ldapconn");
-        IConfigStore authConfig = ldapConfig.getSubStore("ldapauth");
+        LDAPConnectionConfig connConfig = ldapConfig.getConnectionConfig();
+        LDAPAuthenticationConfig authConfig = ldapConfig.getAuthenticationConfig();
 
         LdapConnInfo connInfo = new LdapConnInfo(connConfig);
 
@@ -117,13 +116,15 @@ public class SubsystemDBRemoveCLI extends CommandCLI {
                 connInfo.getPort(),
                 connInfo.getSecure());
 
+        PKISocketConfig socketConfig = cs.getSocketConfig();
+
         PKISocketFactory socketFactory;
         if (authInfo.getAuthType() == LdapAuthInfo.LDAP_AUTHTYPE_SSLCLIENTAUTH) {
             socketFactory = new PKISocketFactory(authInfo.getClientCertNickname());
         } else {
             socketFactory = new PKISocketFactory(connInfo.getSecure());
         }
-        socketFactory.init(cs);
+        socketFactory.init(socketConfig);
 
         LdapBoundConnection conn = new LdapBoundConnection(socketFactory, connInfo, authInfo);
         LDAPConfigurator ldapConfigurator = new LDAPConfigurator(conn, ldapConfig, instanceId);
